@@ -38,6 +38,21 @@ def check_bag_constraint(environ, bag, constraint):
         raise exc.__class__(msg)
 
 
+def check_incoming_etag(environ, etag_string):
+    """
+    Raise 304 if the provided etag is the same as that found in the
+    If-None-Match header of the incoming request.
+
+    Return incoming_etag to indicate that an etag was there but
+    did not match.
+    """
+    incoming_etag = environ.get('HTTP_IF_NONE_MATCH', None)
+    if incoming_etag:
+        if incoming_etag == etag_string:
+            raise HTTP304(incoming_etag)
+    return incoming_etag
+
+
 def check_last_modified(environ, last_modified_string):
     """
     Raise `HTTP304` if If-Modified-Since header matches `last_modified_string`
@@ -141,6 +156,19 @@ def handle_extension(environ, resource_name):
     return resource_name
 
 
+def html_frame(environ, title=''):
+    """
+    Return the header and footer from the current HTML serialization.
+    """
+    # Get HTML framing
+    html = environ.get('tiddlyweb.config', {}).get(
+            'serializers', {}).get('text/html')[0]
+    html = Serializer(html, environ).serialization
+    header = html._header(title)
+    footer = html._footer()
+    return header, footer
+
+
 def http_date_from_timestamp(timestamp):
     """
     Turn a modifier or created tiddler timestamp
